# Uncatched python exception
UNHANDLED_ERROR = 1
# An error occurred while handling the execution command
COMMAND_LINE_ERROR = 2
# An error occurred while executing the game
GAME_EXECUTION_ERROR = 3
# An error occurred while compiling the non-python script
COMPILATION_ERROR = 4

ML_PROCESS_ERROR=100